/*
 *   __               .__       .__  ._____.           
 * _/  |_  _______  __|__| ____ |  | |__\_ |__   ______
 * \   __\/  _ \  \/  /  |/ ___\|  | |  || __ \ /  ___/
 *  |  | (  <_> >    <|  \  \___|  |_|  || \_\ \\___ \ 
 *  |__|  \____/__/\_ \__|\___  >____/__||___  /____  >
 *                   \/       \/             \/     \/ 
 *
 * Copyright (c) 2006-2011 Karsten Schmidt
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * http://creativecommons.org/licenses/LGPL/2.1/
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */

package toxi.geom;

import toxi.math.MathUtils;

/**
 *
 * @author tux
 */
public class XAxisCylinder extends AxisAlignedCylinder {

    /**
     *
     * @param pos
     * @param radius
     * @param length
     */
    public XAxisCylinder(ReadonlyVec3D pos, float radius, float length) {
        super(pos, radius, length);
    }

    @Override
    public boolean containsPoint(ReadonlyVec3D p) {
        if (MathUtils.abs(p.x() - pos.x) < length * 0.5) {
            float dy = p.y() - pos.y;
            float dz = p.z() - pos.z;
            if (Math.abs(dz * dz + dy * dy) < radiusSquared) {
                return true;
            }
        }
        return false;
    }

    @Override
    public Vec3D.Axis getMajorAxis() {
        return Vec3D.Axis.X;
    }
}
